#define BOOST_TEST_MODULE spline_test
#include <boost/test/included/unit_test.hpp>
#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/io.hpp>

#include "../../src/spline.h"

BOOST_AUTO_TEST_CASE( spline1 ) {
	using namespace boost::numeric::ublas;
	using namespace utils::ublas;

	vector< double > x(5);

	x(0)=1;  
	x(1)=2;
	x(2)=4.4;
	x(3)=6;
	x(4)=6.1;
	
	vector< double > y(5);

	y(0)=1.0;
	y(1)=3.0;
	y(2)=-5.0;
	y(3)=9.0;
	y(4)=9.1;
	
	utils::ublas::cubic_spline< vector<double>, vector<double> > q( utils::ublas::make_cubic_spline(x,y) );
	
	for( int i=0;i<x.size();i++ ){
		BOOST_CHECK( y(i) == q(x(i)) );
	}
}

BOOST_AUTO_TEST_CASE( spline2 ) {
	using namespace boost::numeric::ublas;
	using namespace utils::ublas;

	vector< double > x(9);

	x(0)=1;  
	x(1)=2;
	x(2)=3;
	x(3)=4;
	x(4)=5;
	x(5)=6;
	x(6)=7;
	x(7)=8;
	x(8)=9;
	
	vector< double > y(9);

	y(0)=1;  
	y(1)=8;
	y(2)=27;
	y(3)=64;
	y(4)=125;
	y(5)=216;
	y(6)=343;
	y(7)=512;
	y(8)=729;
	
	utils::ublas::cubic_spline< vector<double>, vector<double> > q( utils::ublas::make_cubic_spline(x,y,double(0)) );
	
	for( int i=0;i<x.size();i++ ){
		BOOST_CHECK( y(i) == q(x(i)) );
		boost::tuple<double,double,double> t = q(x(i),true);
		std::cerr << t.get<0>() << " " << t.get<1>() << " " << t.get<2>() << std::endl;
	}
}

//[(1, 1), (2, 8), (3, 27), (4, 64), (5, 125), (6, 216), (7, 343), (8, 512), (9, 729)]
