#ifndef _TRM_DEBUG_H
#define _TRM_DEBUG_H

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/io.hpp>

#include "stop.h"
#include "traits.h"

#ifdef TRM_DEBUG

#define TRM_DEBUG_PRINT_BEGIN() trm::debug::print_begin();
#define TRM_DEBUG_PRINT_END(stop) trm::debug::print_end(stop);
#define TRM_DEBUG_PRINT_FUNCTION(x,value,grad,hesse) trm::debug::print_function(x,value,grad,hesse);
#define TRM_DEBUG_PRINT_INFO(delta,step_norm,step,rho) trm::debug::print_info(delta,step_norm,step,rho);
#else

#define TRM_DEBUG_PRINT_BEGIN()
#define TRM_DEBUG_PRINT_END(stop)
#define TRM_DEBUG_PRINT_FUNCTION(x,value,grad,hesse)
#define TRM_DEBUG_PRINT_INFO(delta,step_norm,step,rho)
#endif

namespace trm {
namespace debug {
	using namespace boost::numeric::ublas;
	
	void print_begin() {
		std::cerr << "trm::debug::print_begin" << std::endl;
	}
	template<class T> void print_end( const trm::stop::basic<T>& stop ) {
		std::cerr << "trm::debug::print_end " << "left_iter[" << stop.iternum() << "] step[" << stop.step_flag() << "] delta[" << stop.delta_flag() << "] grad[" << stop.grad_flag() << "]" << std::endl;
	}
	template<class T> void print_function(const vector<T>& x, T value, const vector<T>& grad, const symmetric_matrix<T>& hesse) {
		std::cerr << "trm::debug::print_function\t" << x << "\t" << value << "\t" << grad << "\t" << hesse << std::endl;
	}
	template<class T> void print_info(T delta, T step_norm, const vector<T>& step, T rho){
		std::cerr << "trm::debug::info\t" << rho << "\t" << delta << "\t" << step_norm << "\t" << step << std::endl;
	}
	
}}

#endif //  _TRM_DEBUG_H
