#ifndef _TRM_STOP_H
#define _TRM_STOP_H

#include "traits.h"

namespace trm {
namespace stop {
namespace detail {

using namespace boost::numeric::ublas;
	
template<class T>
bool small_step( const vector<T>& step, const vector<T>& x, T epsilon_M ) {
	typedef vector<T> vector_type;
	typedef typename vector_type::size_type size_type;

	using std::abs;

	for( size_type i = 0; i < BOOST_UBLAS_SAME(x.size(), step.size()); ++i ) {
		if( abs( step(i) ) > epsilon_M * abs( x(i) ) ) {
			return false;
		}
	}

	return true;
}
template<class T>
bool small_delta( T delta, const vector<T>& x, T epsilon_M) {
	return delta < epsilon_M * norm_2(x);
}
template<class T>
bool small_grad( const vector<T>& grad, const vector<T>& x, T f, T epsilon_M ) {
	typedef vector<T> vector_type;
	typedef typename vector_type::size_type size_type;
	
	using std::abs;
	using std::max;

	if( BOOST_UBLAS_SAME(grad.size(), x.size()) < 1 )
		return true;
	
	for( size_type i = 0; i < BOOST_UBLAS_SAME(grad.size(), x.size()); ++i ) {
		if( abs(grad(i)) * max(1.0,abs(x(i))) / max(1.0,abs(f)) > epsilon_M ) {
			return false;
		}
	}

	return true;
}

}

template<class T>
struct basic {
	typedef T value_type;
	typedef ublas::vector<T> vector_type;

	basic( std::size_t max_iter = 300,
	       const value_type& grad_tol = 10 * std::numeric_limits<T>::epsilon(),
	       const value_type& step_tol = 10 * std::numeric_limits<T>::epsilon(),
	       const value_type& delta_tol = 10 * std::numeric_limits<T>::epsilon() ): iter_(max_iter),
	       grad_tol_(grad_tol),
	       step_tol_(step_tol),
	       delta_tol_(delta_tol),
	       step_flag_(false),delta_flag_(false),grad_flag_(false) {
	}
	
	bool check_iternum() { return --iter_ < 1; }
	std::size_t iternum() const { return iter_; }
	
	bool check_step( const vector_type& step, const vector_type& x ) {
		return step_flag_ = detail::small_step(step,x,step_tol_);
	}
	bool check_delta( const value_type& delta, const vector_type& x ) {
		return delta_flag_ = detail::small_delta(delta,x,delta_tol_);
	}
	bool check_grad( const vector_type& grad, const vector_type& typical_x, const value_type& typical_f ) {
		return grad_flag_ = detail::small_grad(grad,typical_x,typical_f,grad_tol_);
	}
	bool step_flag()  const { return step_flag_; }
	bool delta_flag() const { return delta_flag_; }
	bool grad_flag()  const { return grad_flag_; }
private:
	std::size_t iter_;
	value_type grad_tol_,step_tol_,delta_tol_;
	bool step_flag_,delta_flag_,grad_flag_;
};

}
}

#endif // _TRM_STOP_H
