#ifndef _TEST_QUADRATIC_H
#define _TEST_QUADRATIC_H

#include <boost/tuple/tuple.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/vector.hpp>

namespace ublas = boost::numeric::ublas;

struct quadratic {
	quadratic( const ublas::vector<double>& center ): center_(center) {
	}
	
	boost::tuple< double, ublas::vector<double>, ublas::symmetric_matrix<double> > operator() (const ublas::vector<double>& x) const {
		ublas::vector<double> vec = x - center_;
		ublas::vector<double> ret(vec);
		for( unsigned int i = 0; i < ret.size(); ++i ) {
			ret(i) = vec(i)*vec(i);
		}
		double f=0.0;
		for( unsigned int i = 0; i < ret.size(); ++i ) {
			f += ret(i);
		}
		ublas::vector<double> grad(ret);
		for( unsigned int i = 0; i < ret.size(); ++i ) {
			grad(i) = 2 * vec(i);
		}
		ublas::symmetric_matrix<double> hesse(vec.size(),vec.size());
		for( unsigned int i = 0; i < hesse.size1(); ++i ) {
			for( unsigned int j = 0; j < hesse.size2(); ++j ) {
				if( i != j ) {
					hesse(i,j) = 0.0;
				} else {
					hesse(i,j) = 2.0;
				}
			}
		}
		
		return boost::make_tuple(f,grad,hesse);
	}
private:
	ublas::vector<double> center_;
};

#endif // _TEST_QUADRATIC_H
